@@ -9,39 +9,39 @@ our $JAVAP_EXECUTABLE = 'javap';
 
 use Java::Javap::TypeCast;
 
-sub get_included_types {
+sub get_referenced_typenames {
     shift; # invoked through this class, discard our name
     my $tree     = shift;
-    my $caster   = shift;
    
     my %answers;
 
-    # first, get parent type
-    my $parent   = $tree->{ parent };
-    $answers{ $parent }++ unless ( _skip_it( $parent, $caster ) );
+    for my $type_name ( $tree->{ parent }, @{ $tree->{implements} } ) {
+        $answers{ $type_name }++ if $type_name;
+    }
 
-    # now get types from the children
-    my $contents = $tree->{ contents };
+    foreach my $element ( @{ $tree->{contents} } ) {
 
-    ELEMENT:
-    foreach my $element ( @{ $contents } ) {
-        if ($element->{ body_element } ne 'method') {
-            next ELEMENT;
-        }
+        $answers{ $element->{type}->{name} }++ if $element->{type};
 
-        foreach my $item ( $element->{ type }, @{ $element->{ args } }) {
-            my $type_name = $item->{ name };
-            next if _skip_it( $type_name, $caster );
-
-            $answers{ $type_name }++
-                unless $type_name eq $tree->{java_qualified_name};
-            #warn "Noted $element->{name} type $type_name\n";
+        if (my $args = $element->{args}) {
+            $answers{ $_->{name} }++ for @$args;
         }
     }
 
-    return [ keys %answers ];
+    # remove own class
+    delete $answers{ $tree->{java_qualified_name} };
+    delete $answers{void};
+
+    return keys %answers;
 }
 
+sub get_included_types { # XXX change name
+    my ($class, $tree, $caster) = @_;
+    my @types = $class->get_referenced_typenames($tree);
+    return [ grep { not $caster->defined_cast($_) } @types ];
+}
+
+
 sub invoke_javap {
 	my ($self, $classes, $options) = @_;
 
@@ -82,25 +82,6 @@ sub javap_test {
 	return $output ? 1 : 0;
 }
 
-# Returns true if type casts to a builtin perl6 type
-sub _skip_it {
-    my $type_name    = shift;
-    my $caster  = shift;
-    
-    return 1 if not defined $type_name;
-    return 1 if $type_name eq 'void';
-
-    my $cast = $caster->cast( $type_name );
-    $cast    =~ s/::/./g;
-
-    my $skip_it = 0;
-
-    $skip_it++ if ( $type_name ne $cast );
-    #warn "Skipping $type_name ($cast)\n" if $skip_it;
-
-    return $skip_it;
-}
-
 1;
 
 =head1 NAME
